{
  POP3 Mail Checker

  Message class definition

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit MC_Mail;

interface

uses
  SysUtils, Classes;

const

{ Message headers }

  SFrom = 'From:';
  SSubject = 'Subject:';
  SDate = 'Date:';
  SMessageID = 'Message-ID:';

type

{ TMailMessage }

  TMailMessage = class(TPersistent)
  private
    FMsgNum: Integer;
    FMsgSize: Integer;
    FText: string;
    FFullSource: Boolean;
    function GetDate: string;
    function GetFrom: string;
    function GetMessageID: string;
    function GetSubject: string;
  public
    constructor Create(AMsgNum, AMsgSize: Integer; AText: string);
    procedure Assign(Source: TPersistent); override;
    property Date: string read GetDate;
    property From: string read GetFrom;
    property FullSource: Boolean read FFullSource write FFullSource;
    property MessageID: string read GetMessageID;
    property MsgNum: Integer read FMsgNum;
    property MsgSize: Integer read FMsgSize;
    property Subject: string read GetSubject;
    property Text: string read FText write FText;
  end;

{ TMailBox }

  TMailBox = class(TPersistent)
  private
    FMessages: TList;
    function GetCount: Integer;
    function GetMailMessage(Index: Integer): TMailMessage;
  public
    constructor Create;
    destructor Destroy; override;
    function Add(MsgNum, MsgSize: Integer; const Text: string): Integer;
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    procedure Delete(Index: Integer);
    function IndexOfMsg(MsgNum: Integer): Integer;
    function IndexOfMessageID(const MessageID: string): Integer;
    procedure UpdateMsgNums;
    property Count: Integer read GetCount;
    property Messages[Index: Integer]: TMailMessage read GetMailMessage; default;
  end;

function ExtractMessageHeader(const Message, Header: string): string;

function ExtractMessageHeaders(const Message: string): string;
function ExtractMessageBody(const Message: string): string;

implementation

uses
  MIME;

function ExtractMessageHeader(const Message, Header: string): string;

  function UpcasePos(Substr: string; S: string): Integer;
  begin
    Substr := UpperCase(Substr);
    S := UpperCase(S);
    Result := Pos(Substr, S);
  end;

var
  P, H: Integer;
  S: string;
begin
  Result := '';
  S := Message;
  P := 0;
  { Find header }
  repeat
    H := P;
    if H <> 0 then
    begin
      if (H = 1) or (S[H - 1] in [#13, #10]) then
      begin
        H := H + Length(Header);
        SetString(Result, PChar(S) + H - 1, Length(S) - H + 1);
        H := Pos(#13#10, Result);
        if H <> 0 then SetLength(Result, H - 1);
        Break;
      end
      else
      begin
        H := H + Length(Header);
        SetString(S, PChar(S) + H - 1, Length(S) - H + 1);
      end;
    end; 
    P := UpcasePos(Header, S);
  until P = 0;
  { Encode header }
  Result := DecodeLine7Bit(Trim(Result));
end;

function ExtractMessageHeaders(const Message: string): string;
var
  P: Integer;
begin
  P := Pos(#13#10#13#10, Message);
  if P <> 0 then
    Result := Copy(Message, 1, P - 1) else
    Result := Message;
end;

function ExtractMessageBody(const Message: string): string;
var
  P: Integer;
begin
  P := Pos(#13#10#13#10, Message);
  if P <> 0 then
    Result := Copy(Message, P + 4, Length(Message)) else
    Result := Message;
end;

{ TMailMessage }

constructor TMailMessage.Create(AMsgNum, AMsgSize: Integer; AText: string);
begin
  FMsgNum := AMsgNum;
  FMsgSize := AMsgSize;
  FText := AText;
end;

procedure TMailMessage.Assign(Source: TPersistent);
var
  SourceMessage: TMailMessage absolute Source;
begin
  if Source is TMailMessage then
  begin
    Text := SourceMessage.Text;
    Exit;
  end;
  inherited Assign(Source);
end;

function TMailMessage.GetDate: string;
begin
  Result := ExtractMessageHeader(FText, SDate);
end;

function TMailMessage.GetFrom: string;
begin
  Result := ExtractMessageHeader(FText, SFrom);
end;

function TMailMessage.GetMessageID: string;
begin
  Result := ExtractMessageHeader(FText, SMessageID);
end;

function TMailMessage.GetSubject: string;
begin
  Result := ExtractMessageHeader(FText, SSubject);
end;

{ TMailBox }

constructor TMailBox.Create;
begin
  FMessages := TList.Create;
end;

destructor TMailBox.Destroy;
begin
  Clear;
  FMessages.Free;
end;

function TMailBox.GetCount: Integer;
begin
  Result := FMessages.Count;
end;

function TMailBox.GetMailMessage(Index: Integer): TMailMessage;
begin
  Result := FMessages[Index];
end;

function TMailBox.Add(MsgNum, MsgSize: Integer; const Text: string): Integer;
begin
  Result := FMessages.Add(TMailMessage.Create(MsgNum, MsgSize, Text));
end;

procedure TMailBox.Assign(Source: TPersistent);
var
  SourceBox: TMailBox absolute Source;
  I, Index: Integer;
begin
  if Source is TMailBox then
  begin
    Clear;
    for I := 0 to SourceBox.Count - 1 do
    begin
      Index := Add(SourceBox[I].MsgNum, SourceBox[I].MsgSize, SourceBox[I].Text);
      Messages[Index].FullSource := SourceBox[I].FullSource;
    end;
    Exit;
  end;
  inherited Assign(Source);
end;

procedure TMailBox.Clear;
var
  I: Integer;
begin
  I := Count;
  while I > 0 do
  begin
    Dec(I);
    Delete(I);
  end;
end;

procedure TMailBox.Delete(Index: Integer);
begin
  Messages[Index].Free;
  FMessages.Delete(Index);
end;

function TMailBox.IndexOfMsg(MsgNum: Integer): Integer;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if Messages[I].MsgNum = MsgNum then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

function TMailBox.IndexOfMessageID(const MessageID: string): Integer;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if AnsiCompareText(Messages[I].MessageID, MessageID) = 0 then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

procedure TMailBox.UpdateMsgNums;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do Messages[I].FMsgNum := I + 1;
end;

end.
